@@ -715,6 +715,25 @@ impl CpModelBuilder {
         ffi::cp_model_stats(self.proto())
     }
 
+    /// Verifies that the given model satisfies all the properties
+    /// described in the proto comments. Returns an empty string if it is
+    /// the case, otherwise fails at the first error and returns a
+    /// human-readable description of the issue.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use cp_sat::builder::CpModelBuilder;
+    /// # use cp_sat::proto::CpSolverStatus;
+    /// let mut model = CpModelBuilder::default();
+    /// let x = model.new_int_var([(0, -1)]);
+    /// model.maximize(x);
+    /// assert!(!model.validate_cp_model().is_empty());
+    /// ```
+    pub fn validate_cp_model(&self) -> String {
+        ffi::validate_cp_model(self.proto())
+    }
+
     /// Solves the model, and returns the corresponding [proto::CpSolverResponse].
     ///
     /// # Example
@@ -771,6 +790,7 @@ impl BoolVar {
     /// assert_eq!(response.status(), CpSolverStatus::Optimal);
     /// assert!(x.solution_value(&response));
     /// ```
+    #[track_caller]
     pub fn solution_value(self, response: &proto::CpSolverResponse) -> bool {
         if self.0 < 0 {
             use std::ops::Not;
@@ -822,6 +842,7 @@ impl IntVar {
     /// assert_eq!(response.status(), CpSolverStatus::Optimal);
     /// assert_eq!(42, x.solution_value(&response));
     /// ```
+    #[track_caller]
     pub fn solution_value(self, response: &proto::CpSolverResponse) -> i64 {
         if self.0 < 0 {
             1 - self.not().solution_value(response)
@@ -877,6 +898,11 @@ pub struct Constraint(usize);
 ///         _ => expr -= LinearExpr::from([(42, v), (1337, y1)]) + 5,
 ///     }
 /// }
+///
+/// // an iterator of `Into<LinearExpr>` can be collected or extended,
+/// // meaning summing the elements
+/// model.maximize(vars.iter().copied().collect::<LinearExpr>()); // means sum(vars)
+/// expr.extend(vars.iter().map(|&v| (2, v))); // means expr += sum_vars(2 * v)
 /// ```
 #[derive(Clone, Default, Debug)]
 pub struct LinearExpr {
@@ -974,3 +1000,18 @@ impl From<LinearExpr> for proto::LinearExpressionProto {
         }
     }
 }
+
+impl<T: Into<LinearExpr>> std::iter::Extend<T> for LinearExpr {
+    fn extend<I: IntoIterator<Item = T>>(&mut self, iter: I) {
+        for e in iter {
+            *self += e;
+        }
+    }
+}
+impl<T: Into<LinearExpr>> std::iter::FromIterator<T> for LinearExpr {
+    fn from_iter<I: IntoIterator<Item = T>>(iter: I) -> Self {
+        let mut res = LinearExpr::default();
+        res.extend(iter);
+        res
+    }
+}
